@@ -4,9 +4,10 @@ import Criterion.Main (bench, defaultMain, nf)
 import Text.Pandoc.Definition (Pandoc, Inline (Str))
 import Text.Pandoc.Walk (walk)
 import Text.Pandoc.Builder
+import qualified Data.Text as T
 
 main :: IO ()
-main = do
+main =
   defaultMain [
       bench "simple walk" $ nf (walk prependZeroWidthSpace) mydoc
     , bench "walk concatMap" $ nf (walk $ concatMap prependZeroWidthSpace') mydoc
@@ -14,16 +15,16 @@ main = do
     ]
 
 prependZeroWidthSpace :: Inline -> Inline
-prependZeroWidthSpace (Str s) = Str ('\8203' : s)
+prependZeroWidthSpace (Str s) = Str (T.cons '\8203' s)
 prependZeroWidthSpace x = x
 
 prependZeroWidthSpace' :: Inline -> [Inline]
-prependZeroWidthSpace' (Str s) = [Str ('\8203' : s)]
+prependZeroWidthSpace' (Str s) = [Str (T.cons '\8203' s)]
 prependZeroWidthSpace' x = [x]
 
 prependZeroWidthSpace'' :: [Inline] -> [Inline]
 prependZeroWidthSpace'' (Str s : xs) =
-  (Str ('\8203' : s) : prependZeroWidthSpace'' xs)
+  Str (T.cons '\8203' s) : prependZeroWidthSpace'' xs
 prependZeroWidthSpace'' (x : xs) =
   x : prependZeroWidthSpace'' xs
 prependZeroWidthSpace'' [] = []
